@@ -19,6 +19,10 @@
 #include "zend_attributes.h"
 #endif
 
+#if PHP_VERSION_ID < 70200
+#error "The php-ast 1.1 release dropped support for php 7.0-7.1. Use php-ast 1.0.16 instead."
+#endif
+
 #ifndef ZEND_THIS
 #define ZEND_THIS getThis()
 #endif
@@ -27,12 +31,6 @@
 #define ZEND_ARG_INFO_WITH_DEFAULT_VALUE(pass_by_ref, name, default_value) \
 	ZEND_ARG_INFO(pass_by_ref, name)
 #endif
-#if PHP_VERSION_ID < 70200
-#undef ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX
-#define ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(name, return_reference, required_num_args, class_name, allow_null) \
-	static const zend_internal_arg_info name[] = { \
-	   	{ (const char*)(zend_uintptr_t)(required_num_args), ( #class_name ), 0, return_reference, allow_null, 0 },
-#endif
 
 #ifndef ZEND_BEGIN_ARG_WITH_RETURN_OBJ_INFO_EX
 #define ZEND_BEGIN_ARG_WITH_RETURN_OBJ_INFO_EX(name, return_reference, required_num_args, class_name, allow_null) \
@@ -87,16 +85,6 @@
 #define AST_PLUS 261
 #define AST_MINUS 262
 
-/* Define some compatibility constants */
-#if PHP_VERSION_ID < 70100
-# define IS_VOID 18
-# define IS_ITERABLE 19
-# define ZEND_TYPE_NULLABLE (1<<8)
-# define ZEND_ARRAY_SYNTAX_LIST 1
-# define ZEND_ARRAY_SYNTAX_LONG 2
-# define ZEND_ARRAY_SYNTAX_SHORT 3
-#endif
-
 #if PHP_VERSION_ID < 70300
 # define ZEND_BIND_REF 1
 #endif
@@ -124,6 +112,11 @@
 # define IS_NEVER 22
 #endif
 
+#if PHP_VERSION_ID < 80200
+# define ZEND_ENCAPS_VAR_DOLLAR_CURLY (1<<0)
+# define ZEND_ENCAPS_VAR_DOLLAR_CURLY_VAR_VAR (1<<1)
+#endif
+
 /* This contains state of the ast Node creator. */
 typedef struct ast_state_info {
 	zend_long version;
@@ -159,6 +152,7 @@ static const char *class_flags[] = {
 	AST_FLAG(CLASS_INTERFACE),
 	AST_FLAG(CLASS_ANONYMOUS),
 	AST_FLAG(CLASS_ENUM),
+	AST_FLAG(CLASS_READONLY),
 	NULL
 };
 
@@ -174,6 +168,7 @@ static const char *param_flags[] = {
 static const char *type_flags[] = {
 	AST_FLAG(TYPE_NULL),
 	AST_FLAG(TYPE_FALSE),
+	AST_FLAG(TYPE_TRUE),
 	AST_FLAG(TYPE_BOOL),
 	AST_FLAG(TYPE_LONG),
 	AST_FLAG(TYPE_DOUBLE),
@@ -309,9 +304,9 @@ static const char *func_flags[] = {
 	NULL
 };
 
-// Flags of AST_DIM are marked as combinable in case any other flags get added in the future.
 static const char *dim_flags[] = {
 	AST_FLAG(DIM_ALTERNATIVE_SYNTAX),
+	AST_FLAG(ENCAPS_VAR_DOLLAR_CURLY),
 	NULL
 };
 
@@ -321,6 +316,12 @@ static const char *conditional_flags[] = {
 	NULL
 };
 
+static const char *var_flags[] = {
+	AST_FLAG(ENCAPS_VAR_DOLLAR_CURLY_VAR_VAR),
+	AST_FLAG(ENCAPS_VAR_DOLLAR_CURLY),
+	NULL
+};
+
 static const ast_flag_info flag_info[] = {
 	{ AST_NAME, 0, name_flags },
 	{ ZEND_AST_CLASS, 1, class_flags },
@@ -349,6 +350,7 @@ static const ast_flag_info flag_info[] = {
 	{ ZEND_AST_TRAIT_ALIAS, 1, modifier_flags },
 	{ ZEND_AST_DIM, 1, dim_flags },
 	{ ZEND_AST_CONDITIONAL, 1, conditional_flags },
+	{ ZEND_AST_VAR, 1, var_flags },
 };
 
 static inline void ast_update_property(zval *object, zend_string *name, zval *value) {
@@ -424,7 +426,8 @@ static inline zend_bool ast_kind_uses_attr(zend_ast_kind kind) {
 		|| kind == ZEND_AST_GROUP_USE || kind == ZEND_AST_USE_ELEM
 		|| kind == AST_NAME || kind == AST_CLOSURE_VAR || kind == ZEND_AST_CLASS_CONST_DECL
 		|| kind == ZEND_AST_CLASS_CONST_GROUP
-		|| kind == ZEND_AST_ARRAY || kind == ZEND_AST_DIM || kind == ZEND_AST_CONDITIONAL;
+		|| kind == ZEND_AST_ARRAY || kind == ZEND_AST_DIM || kind == ZEND_AST_CONDITIONAL
+		|| kind == ZEND_AST_VAR;
 }
 
 static inline zend_bool ast_kind_is_decl(zend_ast_kind kind) {
@@ -572,6 +575,7 @@ static const builtin_type_info builtin_types[] = {
 	// {ZEND_STRL("static"), IS_STATIC},  /* Impossible to be parsed before php 8 */
 	{ZEND_STRL("mixed"), IS_MIXED},
 	{ZEND_STRL("never"), IS_NEVER},
+	{ZEND_STRL("true"), IS_TRUE}, /* PHP 8.2 added the true type */
 	{NULL, 0, IS_UNDEF}
 };
 static inline zend_uchar lookup_builtin_type(const zend_string *name) {
@@ -764,16 +768,6 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 			}
 		}
 #endif
-#endif
-		/* This AST_CATCH check should occur before ast_is_name() */
-#if PHP_VERSION_ID < 70100
-		if (ast_kind == ZEND_AST_CATCH && i == 0) {
-			/* Emulate PHP 7.1 format (name list) */
-			zval tmp;
-			ast_create_virtual_node(&tmp, AST_NAME, child->attr, child, state);
-			ast_create_virtual_node_ex(
-					&child_zv, ZEND_AST_NAME_LIST, 0, zend_ast_get_lineno(child), state, 1, &tmp);
-		} else
 #endif
 		if (ast_is_name(child, ast, i)) {
 			ast_name_to_zval(child, ast, &child_zv, i, state);
@@ -803,26 +797,24 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 			/* Skip "uses" child since it is always empty */
 			continue;
 #endif
-#if PHP_VERSION_ID >= 70100
 		} else if (ast_kind == ZEND_AST_LIST && child != NULL) {
 			/* Emulate simple variable list */
 			ast_to_zval(&child_zv, child->child[0], state);
-#else
-		} else if (ast_kind == ZEND_AST_ARRAY
-				&& ast->attr == ZEND_ARRAY_SYNTAX_LIST && child != NULL) {
-			/* Emulate ARRAY_ELEM list */
-			zval ch0, ch1;
-			ast_to_zval(&ch0, child, state);
-			ZVAL_NULL(&ch1);
-			ast_create_virtual_node_ex(
-				&child_zv, ZEND_AST_ARRAY_ELEM, 0, zend_ast_get_lineno(child), state,
-				2, &ch0, &ch1);
-#endif
 		} else {
 			ast_to_zval(&child_zv, child, state);
 		}
-
 		if (child_name) {
+#if PHP_VERSION_ID >= 80200
+			if (ast_kind == ZEND_AST_ARROW_FUNC && i == 2) {
+				/* Imitate the native arrow function ast structure used in php 7.4-8.1 for stmts */
+				/* (This is still different from regular functions, which have AST_STMT_LIST) */
+				/* TODO: In a new node type, remove the ZEND_AST_RETURN node instead. */
+				zval tmp;
+				ZVAL_COPY_VALUE(&tmp, &child_zv);
+				ast_create_virtual_node_ex(
+					&child_zv, ZEND_AST_RETURN, 0, zend_ast_get_lineno(child), state, 1, &tmp);
+			}
+#endif
 			zend_hash_add_new(ht, child_name, &child_zv);
 		} else {
 			zend_hash_next_index_insert(ht, &child_zv);
@@ -830,15 +822,6 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 
 	}
 
-#if PHP_VERSION_ID < 70100
-	/* Emulate docComment on constants, which is not available in PHP 7.0 */
-	if (state->version >= 60 && ast_kind == ZEND_AST_CONST_ELEM) {
-		zval tmp;
-		ZVAL_NULL(&tmp);
-		zend_hash_add_new(ht, AST_STR(str_docComment), &tmp);
-		return;
-	}
-#endif
 #if PHP_VERSION_ID < 80000
 	if (state->version >= 80) {
 		if (ast_kind == ZEND_AST_PARAM) {
@@ -875,7 +858,6 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, ast_state_info_t
 			}
 		}
 #endif
-
 		ZVAL_LONG(&id_zval, state->declIdCounter);
 		state->declIdCounter++;
 		zend_hash_add_new(ht, AST_STR(str___declId), &id_zval);
@@ -1010,14 +992,6 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 #endif
 	}
 
-#if PHP_VERSION_ID < 70100
-	/* Normalize to PHP 7.1 format */
-	if (ast->kind == ZEND_AST_LIST) {
-		ast->kind = ZEND_AST_ARRAY;
-		ast->attr = ZEND_ARRAY_SYNTAX_LIST;
-	}
-#endif
-
 	object_init_ex(zv, ast_node_ce);
 
 	zend_object *obj = Z_OBJ_P(zv);
@@ -1031,8 +1005,14 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 
 	if (ast_kind_is_decl(ast->kind)) {
 		zend_ast_decl *decl = (zend_ast_decl *) ast;
+		uint32_t flags = decl->flags;
+#if PHP_VERSION_ID >= 80200
+		if (ast->kind == ZEND_AST_CLASS) {
+			flags &= ~ZEND_ACC_NO_DYNAMIC_PROPERTIES;
+		}
+#endif
 
-		AST_NODE_SET_PROP_FLAGS(obj, decl->flags);
+		AST_NODE_SET_PROP_FLAGS(obj, flags);
 
 		// This is an undeclared dynamic property and has no cache slot.
 		ast_update_property_long(zv, AST_STR(str_endLineno), decl->end_lineno);
@@ -1055,11 +1035,6 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 
 		zend_hash_add_new(children, AST_STR(str_docComment), &tmp_zv);
 	} else {
-#if PHP_VERSION_ID < 70100
-		if (ast->kind == ZEND_AST_CLASS_CONST_DECL) {
-			ast->attr = ZEND_ACC_PUBLIC;
-		}
-#endif
 		AST_NODE_SET_PROP_FLAGS(obj, ast->attr);
 	}
 
@@ -1100,8 +1075,7 @@ static const zend_long versions[] = {50, 60, 70, 80, 85, 90};
 static const size_t versions_count = sizeof(versions)/sizeof(versions[0]);
 
 static inline zend_bool ast_version_deprecated(zend_long version) {
-	/* Currently no deprecated versions */
-	return 0;
+	return version < 70;
 }
 
 static zend_string *ast_version_info() {
@@ -1445,12 +1419,14 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("CLASS_INTERFACE", ZEND_ACC_INTERFACE);
 	ast_register_flag_constant("CLASS_ANONYMOUS", ZEND_ACC_ANON_CLASS);
 	ast_register_flag_constant("CLASS_ENUM", ZEND_ACC_ENUM);
+	ast_register_flag_constant("CLASS_READONLY", ZEND_ACC_READONLY_CLASS);
 
 	ast_register_flag_constant("PARAM_REF", ZEND_PARAM_REF);
 	ast_register_flag_constant("PARAM_VARIADIC", ZEND_PARAM_VARIADIC);
 
 	ast_register_flag_constant("TYPE_NULL", IS_NULL);
 	ast_register_flag_constant("TYPE_FALSE", IS_FALSE);
+	ast_register_flag_constant("TYPE_TRUE", IS_TRUE);
 	ast_register_flag_constant("TYPE_BOOL", _IS_BOOL);
 	ast_register_flag_constant("TYPE_LONG", IS_LONG);
 	ast_register_flag_constant("TYPE_DOUBLE", IS_DOUBLE);
@@ -1502,15 +1478,9 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("EXEC_REQUIRE", ZEND_REQUIRE);
 	ast_register_flag_constant("EXEC_REQUIRE_ONCE", ZEND_REQUIRE_ONCE);
 
-#if PHP_VERSION_ID >= 70200
 	ast_register_flag_constant("USE_NORMAL", ZEND_SYMBOL_CLASS);
 	ast_register_flag_constant("USE_FUNCTION", ZEND_SYMBOL_FUNCTION);
 	ast_register_flag_constant("USE_CONST", ZEND_SYMBOL_CONST);
-#else
-	ast_register_flag_constant("USE_NORMAL", T_CLASS);
-	ast_register_flag_constant("USE_FUNCTION", T_FUNCTION);
-	ast_register_flag_constant("USE_CONST", T_CONST);
-#endif
 
 	ast_register_flag_constant("MAGIC_LINE", T_LINE);
 	ast_register_flag_constant("MAGIC_FILE", T_FILE);
@@ -1529,6 +1499,9 @@ PHP_MINIT_FUNCTION(ast) {
 
 	ast_register_flag_constant("PARENTHESIZED_CONDITIONAL", ZEND_PARENTHESIZED_CONDITIONAL);
 
+	ast_register_flag_constant("ENCAPS_VAR_DOLLAR_CURLY", ZEND_ENCAPS_VAR_DOLLAR_CURLY);
+	ast_register_flag_constant("ENCAPS_VAR_DOLLAR_CURLY_VAR_VAR", ZEND_ENCAPS_VAR_DOLLAR_CURLY_VAR_VAR);
+
 	INIT_CLASS_ENTRY(tmp_ce, "ast\\Node", class_ast_Node_methods);
 	ast_node_ce = zend_register_internal_class(&tmp_ce);
 	ast_declare_property(ast_node_ce, AST_STR(str_kind), &zv_null);
