@@ -19,6 +19,7 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_TRAIT_ADAPTATIONS,
 	ZEND_AST_USE,
 	ZEND_AST_TYPE_UNION,
+	ZEND_AST_TYPE_INTERSECTION,
 	ZEND_AST_ATTRIBUTE_LIST,
 	ZEND_AST_ATTRIBUTE_GROUP,
 	ZEND_AST_MATCH_ARM_LIST,
@@ -32,6 +33,7 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_CLASS,
 	ZEND_AST_MAGIC_CONST,
 	ZEND_AST_TYPE,
+	ZEND_AST_CALLABLE_CONVERT,
 	ZEND_AST_VAR,
 	ZEND_AST_CONST,
 	ZEND_AST_UNPACK,
@@ -105,6 +107,7 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_CATCH,
 	ZEND_AST_FOR,
 	ZEND_AST_FOREACH,
+	ZEND_AST_ENUM_CASE,
 	ZEND_AST_PARAM,
 };
 
@@ -130,6 +133,7 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_TRAIT_ADAPTATIONS: return "AST_TRAIT_ADAPTATIONS";
 		case ZEND_AST_USE: return "AST_USE";
 		case ZEND_AST_TYPE_UNION: return "AST_TYPE_UNION";
+		case ZEND_AST_TYPE_INTERSECTION: return "AST_TYPE_INTERSECTION";
 		case ZEND_AST_ATTRIBUTE_LIST: return "AST_ATTRIBUTE_LIST";
 		case ZEND_AST_ATTRIBUTE_GROUP: return "AST_ATTRIBUTE_GROUP";
 		case ZEND_AST_MATCH_ARM_LIST: return "AST_MATCH_ARM_LIST";
@@ -143,6 +147,7 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_CLASS: return "AST_CLASS";
 		case ZEND_AST_MAGIC_CONST: return "AST_MAGIC_CONST";
 		case ZEND_AST_TYPE: return "AST_TYPE";
+		case ZEND_AST_CALLABLE_CONVERT: return "AST_CALLABLE_CONVERT";
 		case ZEND_AST_VAR: return "AST_VAR";
 		case ZEND_AST_CONST: return "AST_CONST";
 		case ZEND_AST_UNPACK: return "AST_UNPACK";
@@ -216,6 +221,7 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_CATCH: return "AST_CATCH";
 		case ZEND_AST_FOR: return "AST_FOR";
 		case ZEND_AST_FOREACH: return "AST_FOREACH";
+		case ZEND_AST_ENUM_CASE: return "AST_ENUM_CASE";
 		case ZEND_AST_PARAM: return "AST_PARAM";
 	}
 
@@ -225,11 +231,10 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 	switch (kind) {
 		case AST_NAME:
-			switch (child) {
-				case 0: return AST_STR(str_name);
-			}
-			return NULL;
 		case AST_CLOSURE_VAR:
+		case ZEND_AST_VAR:
+		case ZEND_AST_CONST:
+		case ZEND_AST_LABEL:
 			switch (child) {
 				case 0: return AST_STR(str_name);
 			}
@@ -240,32 +245,8 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 			}
 			return NULL;
 		case ZEND_AST_FUNC_DECL:
-			switch (child) {
-				case 0: return AST_STR(str_params);
-				case 1: return AST_STR(str_uses);
-				case 2: return AST_STR(str_stmts);
-				case 3: return AST_STR(str_returnType);
-				case 4: return AST_STR(str_attributes);
-			}
-			return NULL;
 		case ZEND_AST_CLOSURE:
-			switch (child) {
-				case 0: return AST_STR(str_params);
-				case 1: return AST_STR(str_uses);
-				case 2: return AST_STR(str_stmts);
-				case 3: return AST_STR(str_returnType);
-				case 4: return AST_STR(str_attributes);
-			}
-			return NULL;
 		case ZEND_AST_METHOD:
-			switch (child) {
-				case 0: return AST_STR(str_params);
-				case 1: return AST_STR(str_uses);
-				case 2: return AST_STR(str_stmts);
-				case 3: return AST_STR(str_returnType);
-				case 4: return AST_STR(str_attributes);
-			}
-			return NULL;
 		case ZEND_AST_ARROW_FUNC:
 			switch (child) {
 				case 0: return AST_STR(str_params);
@@ -281,117 +262,37 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 1: return AST_STR(str_implements);
 				case 2: return AST_STR(str_stmts);
 				case 3: return AST_STR(str_attributes);
+				case 4: return AST_STR(str_type);
 			}
 			return NULL;
 		case ZEND_AST_MAGIC_CONST:
-			return NULL;
 		case ZEND_AST_TYPE:
-			return NULL;
-		case ZEND_AST_VAR:
-			switch (child) {
-				case 0: return AST_STR(str_name);
-			}
-			return NULL;
-		case ZEND_AST_CONST:
-			switch (child) {
-				case 0: return AST_STR(str_name);
-			}
+		case ZEND_AST_CALLABLE_CONVERT:
 			return NULL;
 		case ZEND_AST_UNPACK:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_CAST:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_EMPTY:
+		case ZEND_AST_SHELL_EXEC:
+		case ZEND_AST_CLONE:
+		case ZEND_AST_EXIT:
+		case ZEND_AST_PRINT:
+		case ZEND_AST_INCLUDE_OR_EVAL:
+		case ZEND_AST_UNARY_OP:
+		case ZEND_AST_YIELD_FROM:
+		case ZEND_AST_RETURN:
+		case ZEND_AST_ECHO:
+		case ZEND_AST_THROW:
 			switch (child) {
 				case 0: return AST_STR(str_expr);
 			}
 			return NULL;
 		case ZEND_AST_ISSET:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
-		case ZEND_AST_SHELL_EXEC:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_CLONE:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_EXIT:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_PRINT:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_INCLUDE_OR_EVAL:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_UNARY_OP:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_PRE_INC:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
 		case ZEND_AST_PRE_DEC:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
 		case ZEND_AST_POST_INC:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
 		case ZEND_AST_POST_DEC:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
-		case ZEND_AST_YIELD_FROM:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_GLOBAL:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
 		case ZEND_AST_UNSET:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-			}
-			return NULL;
-		case ZEND_AST_RETURN:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_LABEL:
-			switch (child) {
-				case 0: return AST_STR(str_name);
-			}
-			return NULL;
 		case ZEND_AST_REF:
 			switch (child) {
 				case 0: return AST_STR(str_var);
@@ -402,26 +303,12 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 0: return AST_STR(str_offset);
 			}
 			return NULL;
-		case ZEND_AST_ECHO:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
-		case ZEND_AST_THROW:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_GOTO:
 			switch (child) {
 				case 0: return AST_STR(str_label);
 			}
 			return NULL;
 		case ZEND_AST_BREAK:
-			switch (child) {
-				case 0: return AST_STR(str_depth);
-			}
-			return NULL;
 		case ZEND_AST_CONTINUE:
 			switch (child) {
 				case 0: return AST_STR(str_depth);
@@ -445,11 +332,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 			}
 			return NULL;
 		case ZEND_AST_PROP:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-				case 1: return AST_STR(str_prop);
-			}
-			return NULL;
 		case ZEND_AST_NULLSAFE_PROP:
 			switch (child) {
 				case 0: return AST_STR(str_expr);
@@ -475,17 +357,7 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 			}
 			return NULL;
 		case ZEND_AST_ASSIGN:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-				case 1: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_ASSIGN_REF:
-			switch (child) {
-				case 0: return AST_STR(str_var);
-				case 1: return AST_STR(str_expr);
-			}
-			return NULL;
 		case ZEND_AST_ASSIGN_OP:
 			switch (child) {
 				case 0: return AST_STR(str_var);
@@ -499,12 +371,14 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 			}
 			return NULL;
 		case ZEND_AST_ARRAY_ELEM:
+		case ZEND_AST_YIELD:
 			switch (child) {
 				case 0: return AST_STR(str_value);
 				case 1: return AST_STR(str_key);
 			}
 			return NULL;
 		case ZEND_AST_NEW:
+		case ZEND_AST_ATTRIBUTE:
 			switch (child) {
 				case 0: return AST_STR(str_class);
 				case 1: return AST_STR(str_args);
@@ -516,12 +390,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 1: return AST_STR(str_class);
 			}
 			return NULL;
-		case ZEND_AST_YIELD:
-			switch (child) {
-				case 0: return AST_STR(str_value);
-				case 1: return AST_STR(str_key);
-			}
-			return NULL;
 		case ZEND_AST_STATIC:
 			switch (child) {
 				case 0: return AST_STR(str_var);
@@ -529,6 +397,10 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 			}
 			return NULL;
 		case ZEND_AST_WHILE:
+		case ZEND_AST_IF_ELEM:
+		case ZEND_AST_SWITCH:
+		case ZEND_AST_SWITCH_CASE:
+		case ZEND_AST_MATCH:
 			switch (child) {
 				case 0: return AST_STR(str_cond);
 				case 1: return AST_STR(str_stmts);
@@ -540,24 +412,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 1: return AST_STR(str_cond);
 			}
 			return NULL;
-		case ZEND_AST_IF_ELEM:
-			switch (child) {
-				case 0: return AST_STR(str_cond);
-				case 1: return AST_STR(str_stmts);
-			}
-			return NULL;
-		case ZEND_AST_SWITCH:
-			switch (child) {
-				case 0: return AST_STR(str_cond);
-				case 1: return AST_STR(str_stmts);
-			}
-			return NULL;
-		case ZEND_AST_SWITCH_CASE:
-			switch (child) {
-				case 0: return AST_STR(str_cond);
-				case 1: return AST_STR(str_stmts);
-			}
-			return NULL;
 		case ZEND_AST_DECLARE:
 			switch (child) {
 				case 0: return AST_STR(str_declares);
@@ -627,18 +481,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 1: return AST_STR(str_uses);
 			}
 			return NULL;
-		case ZEND_AST_ATTRIBUTE:
-			switch (child) {
-				case 0: return AST_STR(str_class);
-				case 1: return AST_STR(str_args);
-			}
-			return NULL;
-		case ZEND_AST_MATCH:
-			switch (child) {
-				case 0: return AST_STR(str_cond);
-				case 1: return AST_STR(str_stmts);
-			}
-			return NULL;
 		case ZEND_AST_MATCH_ARM:
 			switch (child) {
 				case 0: return AST_STR(str_cond);
@@ -652,12 +494,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 			}
 			return NULL;
 		case ZEND_AST_METHOD_CALL:
-			switch (child) {
-				case 0: return AST_STR(str_expr);
-				case 1: return AST_STR(str_method);
-				case 2: return AST_STR(str_args);
-			}
-			return NULL;
 		case ZEND_AST_NULLSAFE_METHOD_CALL:
 			switch (child) {
 				case 0: return AST_STR(str_expr);
@@ -709,6 +545,14 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 3: return AST_STR(str_stmts);
 			}
 			return NULL;
+		case ZEND_AST_ENUM_CASE:
+			switch (child) {
+				case 0: return AST_STR(str_name);
+				case 1: return AST_STR(str_expr);
+				case 2: return AST_STR(str_docComment);
+				case 3: return AST_STR(str_attributes);
+			}
+			return NULL;
 		case ZEND_AST_PARAM:
 			switch (child) {
 				case 0: return AST_STR(str_type);
@@ -742,6 +586,7 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TRAIT_ADAPTATIONS", ZEND_AST_TRAIT_ADAPTATIONS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_USE", ZEND_AST_USE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TYPE_UNION", ZEND_AST_TYPE_UNION, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_TYPE_INTERSECTION", ZEND_AST_TYPE_INTERSECTION, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ATTRIBUTE_LIST", ZEND_AST_ATTRIBUTE_LIST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ATTRIBUTE_GROUP", ZEND_AST_ATTRIBUTE_GROUP, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_MATCH_ARM_LIST", ZEND_AST_MATCH_ARM_LIST, CONST_CS | CONST_PERSISTENT);
@@ -755,6 +600,7 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLASS", ZEND_AST_CLASS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_MAGIC_CONST", ZEND_AST_MAGIC_CONST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TYPE", ZEND_AST_TYPE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_CALLABLE_CONVERT", ZEND_AST_CALLABLE_CONVERT, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_VAR", ZEND_AST_VAR, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CONST", ZEND_AST_CONST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_UNPACK", ZEND_AST_UNPACK, CONST_CS | CONST_PERSISTENT);
@@ -828,5 +674,6 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CATCH", ZEND_AST_CATCH, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_FOR", ZEND_AST_FOR, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_FOREACH", ZEND_AST_FOREACH, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_ENUM_CASE", ZEND_AST_ENUM_CASE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_PARAM", ZEND_AST_PARAM, CONST_CS | CONST_PERSISTENT);
 }
