@@ -143,10 +143,12 @@ const CLASS_TRAIT = 2;
 const CLASS_INTERFACE = 1;
 const CLASS_ANONYMOUS = 4;
 const CLASS_ENUM = 268435456;
+const CLASS_READONLY = 65536;
 const PARAM_REF = 8;
 const PARAM_VARIADIC = 16;
 const TYPE_NULL = 1;
 const TYPE_FALSE = 2;
+const TYPE_TRUE = 3;
 const TYPE_BOOL = 18;
 const TYPE_LONG = 4;
 const TYPE_DOUBLE = 5;
@@ -210,6 +212,8 @@ const ARRAY_SYNTAX_LONG = 2;
 const ARRAY_SYNTAX_SHORT = 3;
 const DIM_ALTERNATIVE_SYNTAX = 2;
 const PARENTHESIZED_CONDITIONAL = 1;
+const ENCAPS_VAR_DOLLAR_CURLY = 1;
+const ENCAPS_VAR_DOLLAR_CURLY_VAR_VAR = 2;
 // END AST FLAG CONSTANTS
 
 namespace ast;
